"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HttpApi = void 0;
let url = require('url');
const _ = require("lodash");
class HttpApi {
    constructor(io, channel, express, options) {
        this.io = io;
        this.channel = channel;
        this.express = express;
        this.options = options;
    }
    init() {
        this.corsMiddleware();
        this.express.get('/', (req, res) => this.getRoot(req, res));
        this.express.get('/apps/:appId/status', (req, res) => this.getStatus(req, res));
        this.express.get('/apps/:appId/channels', (req, res) => this.getChannels(req, res));
        this.express.get('/apps/:appId/channels/:channelName', (req, res) => this.getChannel(req, res));
        this.express.get('/apps/:appId/channels/:channelName/users', (req, res) => this.getChannelUsers(req, res));
    }
    corsMiddleware() {
        if (this.options.allowCors) {
            this.express.use((req, res, next) => {
                res.header('Access-Control-Allow-Origin', this.options.allowOrigin);
                res.header('Access-Control-Allow-Methods', this.options.allowMethods);
                res.header('Access-Control-Allow-Headers', this.options.allowHeaders);
                next();
            });
        }
    }
    getRoot(req, res) {
        res.send('OK');
    }
    getStatus(req, res) {
        res.json({
            subscription_count: this.io.engine.clientsCount,
            uptime: process.uptime(),
            memory_usage: process.memoryUsage(),
        });
    }
    getChannels(req, res) {
        let prefix = url.parse(req.url, true).query.filter_by_prefix;
        let channels = {};
        let rooms = this.io.sockets.adapter.rooms;
        let sockets = this.io.sockets.sockets;
        rooms.forEach((socketsInRoom, channelName) => {
            if (sockets.has(channelName)) {
                return;
            }
            if (prefix && !channelName.startsWith(prefix)) {
                return;
            }
            channels[channelName] = {
                subscription_count: socketsInRoom.size,
                occupied: true,
            };
        });
        res.json({ channels: channels });
    }
    getChannel(req, res) {
        var _a;
        let channelName = req.params.channelName;
        (_a = this.io.of('/').adapter.rooms.get(channelName)) === null || _a === void 0 ? void 0 : _a.size;
        let room = this.io.sockets.adapter.rooms.get(channelName);
        let subscriptionCount = room ? room.size : 0;
        let result = {
            subscription_count: subscriptionCount,
            occupied: !!subscriptionCount,
        };
        if (this.channel.isPresence(channelName)) {
            this.channel.presence.getMembers(channelName).then((members) => {
                result.user_count = _.uniqBy(members, 'user_id').length;
                res.json(result);
            });
        }
        else {
            res.json(result);
        }
    }
    getChannelUsers(req, res) {
        let channelName = req.params.channelName;
        if (!this.channel.isPresence(channelName)) {
            return this.badResponse(req, res, 'User list is only possible for Presence Channels');
        }
        this.channel.presence.getMembers(channelName).then((members) => {
            let users = _.uniqBy(members, 'user_id').map((member) => ({
                id: member.user_id,
                user_info: member.user_info,
            }));
            res.json({ users: users });
        }).catch((error) => {
            res.status(500).json({ error: 'Internal server error' });
        });
        return true;
    }
    badResponse(req, res, message) {
        res.statusCode = 400;
        res.json({ error: message });
        return false;
    }
}
exports.HttpApi = HttpApi;
