"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Server = void 0;
var fs = require('fs');
var http = require('http');
var https = require('https');
var express = require('express');
var url = require('url');
var io = require('socket.io');
const log_1 = require("./log");
class Server {
    constructor(options) {
        this.options = options;
    }
    init() {
        return new Promise((resolve, reject) => {
            this.serverProtocol().then(() => {
                let host = this.options.host || 'localhost';
                log_1.Log.success(`Running at ${host} on port ${this.getPort()}`);
                resolve(this.io);
            }, error => reject(error));
        });
    }
    getPort() {
        let portRegex = /([0-9]{2,5})[\/]?$/;
        let portToUse = String(this.options.port).match(portRegex);
        if (portToUse && portToUse[1]) {
            return Number(portToUse[1]);
        }
        throw new Error("Invalid port format.");
    }
    serverProtocol() {
        return new Promise((resolve, reject) => {
            if (this.options.protocol == 'https') {
                this.secure().then(() => {
                    resolve(this.httpServer(true));
                }, error => reject(error));
            }
            else {
                resolve(this.httpServer(false));
            }
        });
    }
    secure() {
        return new Promise((resolve, reject) => {
            if (!this.options.sslCertPath || !this.options.sslKeyPath) {
                reject('SSL paths are missing in server config.');
            }
            Object.assign(this.options, {
                cert: fs.readFileSync(this.options.sslCertPath),
                key: fs.readFileSync(this.options.sslKeyPath),
                ca: (this.options.sslCertChainPath) ? fs.readFileSync(this.options.sslCertChainPath) : '',
                passphrase: this.options.sslPassphrase,
            });
            resolve(this.options);
        });
    }
    httpServer(secure) {
        this.express = express();
        this.express.use((req, res, next) => {
            for (var header in this.options.headers) {
                res.setHeader(header, this.options.headers[header]);
            }
            next();
        });
        var httpServer = secure
            ? https.createServer(this.options, this.express)
            : http.createServer(this.express);
        httpServer.listen(this.getPort(), this.options.host);
        this.authorizeRequests();
        return this.io = io(httpServer, this.options.socketio);
    }
    authorizeRequests() {
        this.express.param('appId', (req, res, next) => {
            if (!this.canAccess(req)) {
                return this.unauthorizedResponse(req, res);
            }
            next();
        });
    }
    canAccess(req) {
        let appId = this.getAppId(req);
        let key = this.getAuthKey(req);
        if (key && appId) {
            let client = this.options.clients.find((client) => {
                return client.appId === appId;
            });
            if (client) {
                return client.key === key;
            }
        }
        return false;
    }
    getAppId(req) {
        if (req.params.appId) {
            return req.params.appId;
        }
        return false;
    }
    getAuthKey(req) {
        if (req.headers.authorization) {
            return req.headers.authorization.replace('Bearer ', '');
        }
        if (url.parse(req.url, true).query.auth_key) {
            return url.parse(req.url, true).query.auth_key;
        }
        return false;
    }
    unauthorizedResponse(req, res) {
        res.statusCode = 403;
        res.json({ error: 'Unauthorized' });
        return false;
    }
}
exports.Server = Server;
